@@ -1,4 +1,4 @@
-use git2::{Commit, Oid, Repository, Sort};
+use git2::{Commit, Repository, Sort};
 use git_whennes::error::{Error, Result};
 use pico_args::Arguments;
 
@@ -46,12 +46,16 @@ fn main() -> Result<()> {
     if commit_to_check.parent_count() > 1 {
         log::warn!("Merge commit");
         show_commit(commit_to_check)?;
-        // TODO(nds): This is what git-whence exists with. Does it really make sense? Merge commits are
-        //  okay.
+        // TODO(nds): This is what git-whence exits with. Does it really make
+        //  sense? Merge commits are okay.
         std::process::exit(1);
     }
 
     let main_branch = if let Some(main_branch) = args.main_branch {
+        log::debug!(
+            "Using '{}' as main branch, specified by flag",
+            main_branch
+        );
         main_branch
     } else {
         let config = repo.config()?;
@@ -59,9 +63,15 @@ fn main() -> Result<()> {
         // git2 doesn't like it because the config is 'live' (not readonly):
         // https://github.com/libgit2/libgit2/blob/4bd172087c30e09e7720a7df11cace47ee002256/src/config.c#L872-L875.
         let entry = config.get_entry("init.defaultbranch")?;
-        entry.value().unwrap_or("master").into()
+        let main_branch = entry.value().unwrap_or("master").into();
+        log::debug!("Using '{}' as main branch, determined from git config with fallback to 'master'", main_branch);
+        main_branch
     };
-    log::debug!("Using '{}' as the main branch", main_branch);
+
+    log::debug!(
+        "Finding merge commit for {} using strategy 'simple'",
+        args.sha
+    );
     let merge_commit = find_merge_simple(&repo, &args.sha, &main_branch)?;
     if cfg!(feature = "forge") && (args.show_url || args.open_url) {
         use git_whennes::forge;
@@ -89,7 +99,7 @@ fn main() -> Result<()> {
 fn show_commit(commit: Commit) -> Result<()> {
     let summary_line: &str = commit
         .message()
-        .map(|msg| msg.lines().nth(0).unwrap_or_default())
+        .map(|msg| msg.lines().next().unwrap_or_default())
         .unwrap_or_default();
     let obj = commit.as_object();
     let short_id = obj.short_id()?;
@@ -97,8 +107,16 @@ fn show_commit(commit: Commit) -> Result<()> {
     Ok(())
 }
 
-fn get_commit<'repo>(repo: &'repo Repository, sha: &str) -> Result<Commit<'repo>> {
-    let obj = repo.revparse_single(sha)?;
+fn get_commit<'repo>(
+    repo: &'repo Repository,
+    sha: &str,
+) -> Result<Commit<'repo>> {
+    let obj = repo
+        .revparse_single(sha)
+        .map_err(|e| Error::RevParseError {
+            sha: sha.to_string(),
+            source: e,
+        })?;
     let commit = obj.peel_to_commit()?;
     Ok(commit)
 }
@@ -116,17 +134,52 @@ fn find_merge_simple<'repo>(
     from: &str,
     to: &str,
 ) -> Result<Commit<'repo>> {
-    let from_commit = repo.find_commit(Oid::from_str(from)?)?;
-    let spec = format!("{}..{}", from, to);
+    let from_object =
+        repo.revparse_single(from)
+            .map_err(|e| Error::RevParseError {
+                sha: from.to_string(),
+                source: e,
+            })?;
+    let from_commit =
+        from_object.as_commit().ok_or(Error::ExpectedCommitError {
+            sha: from.to_string(),
+        })?;
+    let to_object =
+        repo.revparse_single(to).map_err(|e| Error::RevParseError {
+            sha: to.to_string(),
+            source: e,
+        })?;
+    let to_commit =
+        to_object.as_commit().ok_or(Error::ExpectedCommitError {
+            sha: to.to_string(),
+        })?;
+    let spec = format!(
+        "{}..{}",
+        from_commit.id().to_string(),
+        to_commit.id().to_string()
+    );
     let mut revwalk = repo.revwalk()?;
     revwalk.simplify_first_parent()?;
     revwalk.set_sorting(Sort::REVERSE)?;
     revwalk.push_range(&spec)?;
+    log::debug!("Walking revisions for spec {} in reverse order", spec);
     for id in revwalk {
         // TODO(nds): When would this fail?
         let id = id?;
-        let commit = repo.find_commit(id)?;
-        if commit.parent_count() > 1 && repo.graph_descendant_of(id, from_commit.id())? {
+        let commit =
+            repo.find_commit(id).map_err(|e| Error::FindCommitError {
+                sha: id.to_string(),
+                source: e,
+            })?;
+        if commit.parent_count() > 1
+            && repo
+                .graph_descendant_of(id, from_commit.id())
+                .map_err(|e| Error::GraphDescendantOfError {
+                    commit: id.to_string(),
+                    ancestory: from_commit.id().to_string(),
+                    source: e,
+                })?
+        {
             return Ok(commit);
         }
     }
@@ -155,7 +208,7 @@ fn parse_args() -> Result<Args> {
         #[cfg(feature = "forge")]
         remote: args
             .opt_value_from_str(["-r", "--remote"])?
-            .unwrap_or("origin".into()),
+            .unwrap_or_else(|| "origin".into()),
         sha: args.free_from_str()?,
     })
 }
